@@ -2,16 +2,24 @@ from reapy_boost.errors import DisconnectedClientError, DistError
 from reapy_boost.tools import json
 from .socket import Socket
 
+from ipaddress import IPv4Address, ip_address
+
+LOCALHOST = "127.0.0.1"
+def normalize_localhost(host:IPv4Address) -> str:
+    if host == IPv4Address(LOCALHOST):
+        return 'localhost'
+    return str(host)
 
 class Client(Socket):
 
-    def __init__(self, port, host="localhost"):
+    def __init__(self, port: int, host: IPv4Address = IPv4Address(LOCALHOST)):
         super().__init__()
+        host = normalize_localhost(host)
         self._connect(port, host)
         self.port, self.host = port, host
 
-    def _connect(self, port, host):
-        super().connect((host, port))
+    def _connect(self, port: int, host: IPv4Address):
+        super().connect((str(host), port))
         self.address = self.recv(timeout=None).decode("ascii")
 
     def _get_result(self):
